# frozen_string_literal: true

class PostReceive # rubocop:disable Scalability/IdempotentWorker
  include ApplicationWorker

  feature_category :source_code_management
  urgency :high
  worker_resource_boundary :cpu
  weight 5
  loggable_arguments 0, 1, 2, 3

  def perform(gl_repository, identifier, hg_prunes, changes, push_options = {})
    container, project, repo_type = Gitlab::GlRepository.parse(gl_repository)

    if project.nil? && (!repo_type.snippet? || container.is_a?(ProjectSnippet))
      log("Triggered hook for non-existing project with gl_repository \"#{gl_repository}\"")
      return false
    end

    changes = Base64.decode64(changes) unless changes.include?(' ')
    # Use Sidekiq.logger so arguments can be correlated with execution
    # time and thread ID's.
    Sidekiq.logger.info "changes: #{changes.inspect}" if ENV['SIDEKIQ_LOG_ARGUMENTS']
    post_received = Gitlab::GitPostReceive.new(container, identifier, changes, push_options)

    if repo_type.project? && project.mercurial? && !hg_prunes.nil?
      hg_process_project_prunes(container, post_received, hg_prunes)
    end

    if repo_type.wiki?
      process_wiki_changes(post_received, container)
    elsif repo_type.project?
      process_project_changes(post_received, container)
    elsif repo_type.snippet?
      process_snippet_changes(post_received, container)
    else
      # Other repos don't have hooks for now
    end
  end

  private

  def identify_user(post_received)
    post_received.identify.tap do |user|
      log("Triggered hook for non-existing user \"#{post_received.identifier}\"") unless user
    end
  end

  def hg_process_project_prunes(project, post_received, prunes)
    user = identify_user(post_received)

    logprefix = "PostReceive.hg_process_project_prunes for Project id=#{project.id}"
    prunes = JSON.parse(prunes)

    published_topics = prunes["topic_published"]
    unless published_topics.nil?
      published_topics.each_pair do |gl_branch, latest_sha|
        gl_branch = base64_to_utf8(gl_branch)
        Rails.logger.info("#{logprefix} looking for MRs from published topic '#{gl_branch}'")
        hg_branch, _topic = Gitlab::Mercurial::parse_gitlab_branch_for_hg(gl_branch)
        # locate merge request with topic as source.
        # The odds are that generally, a MR for a published topic will target the
        # corresponding named branch. Therefore is most cases, we won't need to
        # preload the MR diffs like GitLab's standard detection does: we'll reach the
        # conclusion without them. Only if the MR is merged will we need the diff,
        # for potential last update to `latest_sha`.

        # TODO OPTIM make just one SQL request for all published topics
        # and then, sort/match them (could be defeated by the way ActiveRecord
        # fetches from the DB, though).
        # MRs being merged from the Rails app are *not* considered, as
        # they are in the 'locked' state.
        merged = project.merge_requests.opened
          .where(source_branch: gl_branch, source_project: project).to_a
          .select do |merge_request|

          if merge_request.target_branch == 'branch/' + hg_branch
            Rails.logger.info(
              "#{logprefix} marking MR iid=#{merge_request.iid} merged: "\
              "source is the published '#{gl_branch}' "\
              "and target is the corresponding named branch")
            true
          else
            real_target_sha = merge_request.target_branch_sha
            # TODO what if previous position is merged and subsequent
            # changes are not (or maybe in another branch?)
            merged = project.repository.ancestor?(latest_sha, real_target_sha)
            Rails.logger.info(
              "#{logprefix} analysis of MR iid=#{merge_request.iid} "\
              "gives merged=#{merged}. source is the published "\
              "'#{gl_branch}', with latest SHA #{latest_sha} "\
              "and target #{merge_request.target_branch} is another named "\
              "branch with current head SHA #{real_target_sha}")
            merged
            # If merged is false, we don't need to close `merge_request`:
            # MergeRequests::RefreshService will take care of that later
            # with all necessary details.
          end
        end

        merged.each do |merge_request|
          MergeRequests::HgManuallyMergedService.new(project, user).execute(merge_request, latest_sha)
        end
      end
    end
    closed_branches = prunes['branch_closed']
    unless closed_branches.nil?
      merged_with_sha = []
      closed_branches.each_pair do |gl_branch, info|
        next if info.nil?

        gl_branch = base64_to_utf8(gl_branch)

        info.each do |closing_sha, parent_shas|
          merged_with_sha.concat(merged_for_closing_changeset(
                                   project, gl_branch,
                                   closing_sha, parent_shas))
        end
      end
      merged_with_sha.each do |merge_request, latest_sha|
        MergeRequests::HgManuallyMergedService.new(project, user).execute(merge_request, latest_sha)
      end
    end
  end

  # Given a closing commit SHA and the SHAs of its parents, find the
  # Merge Requests that have to be updated and merged.
  def merged_for_closing_changeset(project, gl_branch, closing_sha, parent_shas)
    logprefix = "PostReceive analyzing Merge Requests for changeset "\
                "#{closing_sha}, that closes '#{gl_branch}'"
    repo = project.repository
    merged = []
    project.merge_requests.opened
      .where(source_branch: gl_branch, source_project: project)
      .each do |merge_request|
      real_target_sha = merge_request.target_branch_sha
      if repo.ancestor?(closing_sha, real_target_sha)
        merged << [merge_request, closing_sha]
        Rails.logger.info("#{logprefix}: it is a direct ancestor "\
                          "of MR iid=#{merge_request.iid}, targeting "\
                          "'#{merge_request.target_branch}'. "\
                          "Marking as merged.")
      else
        if parent_shas.all? {|psha| repo.ancestor?(psha, real_target_sha)}
          Rails.logger.info("#{logprefix}: all its parents #{parent_shas} "\
                            "are ancestors of MR iid=#{merge_request.iid}, "\
                            "targeting '#{merge_request.target_branch}'. "\
                            "Marking as merged.")
          # if the closing commit has several parents, since we have to update
          # the MR with a single head, let's put in the closing commit to see
          # all of them and be faithful to what happened.
          merged << [merge_request,
                     parent_shas.length == 1? parent_shas[0] : closing_sha]
        end
      end
    end
    merged
  end

  # Deserialize a binary string from Base64, then guess and convert to UTF-8
  #
  # This is the same as what happens for the `changes` param, here (base64)
  # and in `Gitlab::GitPostReceive` (transcoding).
  def base64_to_utf8(s)
    decoded = Base64.decode64(s)
    decoded.force_encoding('UTF-8')
    return decoded if decoded.valid_encoding?

    detection = CharlockHolmes::EncodingDetector.detect(changes)
    return decoded unless detection && detection[:encoding]

    CharlockHolmes::Converter.convert(decoded, detection[:encoding], 'UTF-8')
  end

  def hg_fixup_default_branch(project)
    return if !project.mercurial?
    existing = project.default_branch
    return if existing.nil? || !existing.start_with?('topic/')

    project.repository.expire_root_ref_cache
    new_default = project.reload_default_branch
    unless new_default == existing
      protection = ProtectedBranch.where(project: project, name: existing).first
      protection.destroy unless protection.nil?
    end
  end

  def process_project_changes(post_received, project)
    logprefix = "PostReceive analyzing Merge Requests for Project id=#{project.id} changes=#{post_received.changes}"
    user = identify_user(post_received)

    return false unless user
    push_options = post_received.push_options
    changes = post_received.changes

    # We only need to expire certain caches once per push
    expire_caches(post_received, project.repository)
    hg_fixup_default_branch(project)
    enqueue_project_cache_update(post_received, project)

    process_ref_changes(project, user, push_options: push_options, changes: changes)
    update_remote_mirrors(post_received, project)
    after_project_changes_hooks(project, user, changes.refs, changes.repository_data)
  end

  def process_wiki_changes(post_received, project)
    project.touch(:last_activity_at, :last_repository_updated_at)
    project.wiki.repository.expire_statistics_caches
    ProjectCacheWorker.perform_async(project.id, [], [:wiki_size])

    user = identify_user(post_received)
    return false unless user

    # We only need to expire certain caches once per push
    expire_caches(post_received, project.wiki.repository)

    ::Git::WikiPushService.new(project, user, changes: post_received.changes).execute
  end

  def process_snippet_changes(post_received, snippet)
    user = identify_user(post_received)

    return false unless user

    expire_caches(post_received, snippet.repository)
    Snippets::UpdateStatisticsService.new(snippet).execute
  end

  # Expire the repository status, branch, and tag cache once per push.
  def expire_caches(post_received, repository)
    repository.expire_status_cache if repository.empty?
    repository.expire_branches_cache if post_received.includes_branches?
    repository.expire_caches_for_tags if post_received.includes_tags?
  end

  # Schedule an update for the repository size and commit count if necessary.
  def enqueue_project_cache_update(post_received, project)
    stats_to_invalidate = [:repository_size]
    stats_to_invalidate << :commit_count if post_received.includes_default_branch?

    ProjectCacheWorker.perform_async(
      project.id,
      [],
      stats_to_invalidate,
      true
    )
  end

  def process_ref_changes(project, user, params = {})
    return unless params[:changes].any?

    Git::ProcessRefChangesService.new(project, user, params).execute
  end

  def update_remote_mirrors(post_received, project)
    return unless post_received.includes_branches? || post_received.includes_tags?

    return unless project.has_remote_mirror?

    project.mark_stuck_remote_mirrors_as_failed!
    project.update_remote_mirrors
  end

  def after_project_changes_hooks(project, user, refs, changes)
    repository_update_hook_data = Gitlab::DataBuilder::Repository.update(project, user, changes, refs)
    SystemHooksService.new.execute_hooks(repository_update_hook_data, :repository_update_hooks)
    Gitlab::UsageDataCounters::SourceCodeCounter.count(:pushes)
  end

  def log(message)
    Gitlab::GitLogger.error("POST-RECEIVE: #{message}")
  end
end

PostReceive.prepend_if_ee('EE::PostReceive')
