# frozen_string_literal: true

require 'yaml'
require 'rake'

module Backup
  class Repository
    attr_reader :progress

    def initialize(progress)
      @progress = progress
    end

    # Return [path to hg executable, suitable working directory, environment]
    #
    # for all `hg` commands, we'll use explicit path arguments
    # (better for logging anyway), so any existing directory
    # that's *not* also an hg repo (that excludes Rails root) would do.
    def hg_cmd_details
          [Gitlab.config.mercurial.bin_path,
           backup_repos_path,  # guaranteed to exist and not an hg repo
           { "HGRCPATH" => Gitlab::Mercurial.hgrc_path,
           },
          ]
    end

    def dump(max_concurrency:, max_storage_concurrency:)
      prepare

      if max_concurrency <= 1 && max_storage_concurrency <= 1
        return dump_consecutive
      end

      if Project.excluding_repository_storage(Gitlab.config.repositories.storages.keys).exists?
        raise Error, 'repositories.storages in gitlab.yml is misconfigured'
      end

      semaphore = Concurrent::Semaphore.new(max_concurrency)
      errors = Queue.new

      threads = Gitlab.config.repositories.storages.keys.map do |storage|
        Thread.new do
          Rails.application.executor.wrap do
            dump_storage(storage, semaphore, max_storage_concurrency: max_storage_concurrency)
          rescue => e
            errors << e
          end
        end
      end

      ActiveSupport::Dependencies.interlock.permit_concurrent_loads do
        threads.each(&:join)
      end

      raise errors.pop unless errors.empty?
    end

    def backup_project(project)
      path_to_project_bundle = path_to_bundle(project)

      # TODO make a project method out of this
      has_git = ["git", "hg_git"].include?(project.vcs_type)
      if has_git
        Gitlab::GitalyClient::RepositoryService.new(project.repository)
          .create_bundle(path_to_project_bundle)
        backup_custom_hooks(project)
      end
      if project.mercurial?
        backup_hg_repo(project)
      end
    rescue => e
      progress_warn(project, e, 'Failed to backup repo')
    end

    def backup_hg_repo(project)
      path_to_hg_repo = path_to_hg_repo(project)
      path_to_hg_project_bundle = path_to_hg_bundle(project)
      hg_exe, hg_cwd, hg_env = hg_cmd_details

      FileUtils.mkdir_p(path_to_hg_tars(project))

      # Backup the hg config and necessary caches
      # In Heptapod 0.12, we don't need to backup the `.hg/git`,
      # which is the only binary (platform dependent) thing we have here
      cmd = ['tar',
                 '-cf', path_to_hg_tars(project, "config-and-cache"),
                 '-C', path_to_hg_repo,
                 '--ignore-failed-read',
                 '.hg/hgrc',
                 '.hg/hgrc.managed',
                 '.hg/git-mapfile',
                ]
      output, status = Gitlab::Popen.popen(cmd)
      progress_warn(project, cmd.join(' '), output) unless status.zero?

      # Backup the hg repository itself
      cmd = [hg_exe, "-R", path_to_hg_repo,
             "bundle", "--all", path_to_hg_project_bundle]
      output, status = Gitlab::Popen.popen(cmd, hg_cwd, hg_env)

      unless status.zero?
        progress_warn(project, cmd.join(' '), output)
        raise StandardError, output.strip
      end
    end

    def backup_custom_hooks(project)
      FileUtils.mkdir_p(project_backup_path(project))

      custom_hooks_path = custom_hooks_tar(project)
      Gitlab::GitalyClient::RepositoryService.new(project.repository)
        .backup_custom_hooks(custom_hooks_path)
    end

    def restore_custom_hooks(project)
      return unless Dir.exist?(project_backup_path(project))
      return if Dir.glob("#{project_backup_path(project)}/custom_hooks*").none?

      custom_hooks_path = custom_hooks_tar(project)
      Gitlab::GitalyClient::RepositoryService.new(project.repository)
        .restore_custom_hooks(custom_hooks_path)
    end

    def restore
      Project.find_each(batch_size: 1000) do |project|
        progress.print " * #{display_repo_path(project)} ... "
        # TODO make a project method out of this
        has_git = ["git", "hg_git"].include?(project.vcs_type)

        restore_repo_success =
          begin
            try_restore_repository(project) if has_git
            try_restore_hg_repo(project, !has_git) if project.mercurial?
          rescue => err
            progress.puts "Error: #{err}".color(:red)
            false
          end

        if restore_repo_success
          progress.puts "[DONE]".color(:green)
        else
          progress.puts "[Failed] restoring #{project.full_path} repository".color(:red)
        end

        wiki = ProjectWiki.new(project)
        wiki.repository.remove rescue nil
        path_to_wiki_bundle = path_to_bundle(wiki)
        path_to_hg_wiki_bundle = path_to_hg_bundle(wiki)

        if File.exist?(path_to_wiki_bundle) || File.exist?(path_to_hg_wiki_bundle)
          progress.print " * #{wiki.full_path} ... "
          begin
            if has_git
              wiki.repository.create_from_bundle(path_to_wiki_bundle)
              restore_custom_hooks(wiki)
            end
            if project.mercurial?
              try_restore_hg_repo(wiki, !has_git)
            end
            progress.puts "[DONE]".color(:green)
          rescue => e
            progress.puts "[Failed] restoring #{wiki.full_path} wiki".color(:red)
            progress.puts "Error #{e}".color(:red)
          end
        end
      end

      restore_object_pools
    end

    # param `create_if_missing` is useful in case a Git restoration comes first
    # (happens only in practice if vcs_type == 'hg_git'
    #
    # indeed the first restoration to call `create_repository` would
    # run on HgGitRepository, creating hence both Mercurial and Git repos.
    # we let the caller to decide, since it's also the one decided whether
    # a Git restoration happens first or not.
    def try_restore_hg_repo(project, create_if_missing = true)
      path_to_hg_project_bundle = path_to_hg_bundle(project)
      if !File.exist?(path_to_hg_project_bundle)
        project.repository.create_repository if create_if_missing
        return
      end

      path_to_hg_repo = path_to_hg_repo(project)
      hg_exe, hg_cwd, hg_env = hg_cmd_details

      # Create the Mercurial repository
      cmd = [hg_exe, "init", path_to_hg_repo]
      output, status = Gitlab::Popen.popen(cmd, hg_cwd, hg_env)
      raise StandardError, output.strip unless status.zero?

      # Untar the configuration and necessary caches
      cmd = %W(tar -xf #{path_to_hg_tars(project, "config-and-cache")} -C #{path_to_hg_repo})
      output, status = Gitlab::Popen.popen(cmd)
      raise StandardError, output.strip unless status.zero?

      # Unbundle the backup
      cmd = [hg_exe, "-R", path_to_hg_repo,
             "unbundle", path_to_hg_project_bundle]
      output, status = Gitlab::Popen.popen(cmd, hg_cwd, hg_env)
      raise StandardError, output.strip unless status.zero?
    end


    protected

    def path_to_hg_repo(project)
      project.repository.path_to_repo.ext('.hg')
    end

    def try_restore_repository(project)
      path_to_project_bundle = path_to_bundle(project)
      project.repository.remove rescue nil

      if File.exist?(path_to_project_bundle)
        project.repository.create_from_bundle(path_to_project_bundle)
        restore_custom_hooks(project)
      else
        project.repository.create_repository
      end

      true
    end

    def path_to_bundle(project)
      File.join(backup_repos_path, project.disk_path + '.bundle')
    end

    def path_to_hg_bundle(project)
      File.join(backup_repos_path, project.disk_path + '.hg.bundle')
    end

    def project_backup_path(project)
      File.join(backup_repos_path, project.disk_path)
    end

    def custom_hooks_tar(project)
      File.join(project_backup_path(project), "custom_hooks.tar")
    end

    def path_to_hg_tars(project, dir = nil)
      path = File.join(backup_repos_path, project.disk_path + '.hg')

      if dir
        File.join(path, "#{dir}.tar")
      else
        path
      end
    end

    def backup_repos_path
      File.join(Gitlab.config.backup.path, 'repositories')
    end

    def prepare
      FileUtils.rm_rf(backup_repos_path)
      FileUtils.mkdir_p(Gitlab.config.backup.path)
      FileUtils.mkdir(backup_repos_path, mode: 0700)
    end

    private

    def dump_consecutive
      Project.find_each(batch_size: 1000) do |project|
        dump_project(project)
      end
    end

    def dump_storage(storage, semaphore, max_storage_concurrency:)
      errors = Queue.new
      queue = SizedQueue.new(1)

      threads = Array.new(max_storage_concurrency) do
        Thread.new do
          Rails.application.executor.wrap do
            while project = queue.pop
              semaphore.acquire

              begin
                dump_project(project)
              rescue => e
                errors << e
                break
              ensure
                semaphore.release
              end
            end
          end
        end
      end

      Project.for_repository_storage(storage).find_each(batch_size: 100) do |project|
        break unless errors.empty?

        queue.push(project)
      end

      raise errors.pop unless errors.empty?
    ensure
      queue.close
      ActiveSupport::Dependencies.interlock.permit_concurrent_loads do
        threads.each(&:join)
      end
    end

    def dump_project(project)
      progress.puts " * #{display_repo_path(project)} ... "

      if project.hashed_storage?(:repository)
        FileUtils.mkdir_p(File.dirname(File.join(backup_repos_path, project.disk_path)))
      else
        FileUtils.mkdir_p(File.join(backup_repos_path, project.namespace.full_path)) if project.namespace
      end

      if !empty_repo?(project)
        backup_project(project)
        progress.puts " * #{display_repo_path(project)} ... " + "[DONE]".color(:green)
      else
        progress.puts " * #{display_repo_path(project)} ... " + "[SKIPPED]".color(:cyan)
      end

      wiki = ProjectWiki.new(project)

      if !empty_repo?(wiki)
        backup_project(wiki)
        progress.puts " * #{display_repo_path(project)} ... " + "[DONE] Wiki".color(:green)
      else
        progress.puts " * #{display_repo_path(project)} ... " + "[SKIPPED] Wiki".color(:cyan)
      end
    end

    def progress_warn(project, cmd, output)
      progress.puts "[WARNING] Executing #{cmd}".color(:orange)
      progress.puts "Ignoring error on #{display_repo_path(project)} - #{output}".color(:orange)
    end

    def empty_repo?(project_or_wiki)
      project_or_wiki.repository.expire_emptiness_caches
      project_or_wiki.repository.empty?
    end

    def display_repo_path(project)
      project.hashed_storage?(:repository) ? "#{project.full_path} (#{project.disk_path})" : project.full_path
    end

    def restore_object_pools
      PoolRepository.includes(:source_project).find_each do |pool|
        progress.puts " - Object pool #{pool.disk_path}..."

        pool.source_project ||= pool.member_projects.first.root_of_fork_network
        pool.state = 'none'
        pool.save

        pool.schedule
      end
    end
  end
end
