# frozen_string_literal: true

module Gitlab
  module BitbucketImport
    class Importer
      LABELS = [{ title: 'bug', color: '#FF0000' },
                { title: 'enhancement', color: '#428BCA' },
                { title: 'proposal', color: '#69D100' },
                { title: 'task', color: '#7F8C8D' }].freeze

      attr_reader :project, :client, :errors, :users

      AttachmentError = Class.new(StandardError)

      def initialize(project)
        @project = project
        @client = Bitbucket::Client.new(project.import_data.credentials)
        @formatter = Gitlab::ImportFormatter.new
        @labels = {}
        @errors = []
        @users = {}
      end

      def execute
        import_wiki
        import_issues
        import_pull_requests
        import_repo_config
        handle_errors
        metrics.track_finished_import

        true
      end

      def reimport
        Rails.logger.info("Bitbucket incremental reimport starting for "\
                          "project id=#{project.id}")
        import_issues
        import_pull_requests

        success = errors.empty?
        handle_errors
        success
      end

      private

      def handle_errors
        return unless errors.any?

        project.import_state.update_column(:last_error, {
          message: 'The remote data could not be fully imported.',
          errors: errors
        }.to_json)
      end

      def store_pull_request_error(pull_request, ex)
        backtrace = Gitlab::BacktraceCleaner.clean_backtrace(ex.backtrace)
        error = { type: :pull_request, iid: pull_request.iid, errors: ex.message, trace: backtrace, raw_response: pull_request.raw&.to_json }

        Gitlab::ErrorTracking.log_exception(ex, error)

        # Omit the details from the database to avoid blowing up usage in the error column
        error.delete(:trace)
        error.delete(:raw_response)

        errors << error
      end

      def gitlab_user_id(project, username)
        find_user_id(username) || project.creator_id
      end

      # rubocop: disable CodeReuse/ActiveRecord
      def find_user_id(username)
        return unless username

        return users[username] if users.key?(username)

        users[username] = User.select(:id)
                              .joins(:identities)
                              .find_by("identities.extern_uid = ? AND identities.provider = 'bitbucket'", username)
                              .try(:id)
      end
      # rubocop: enable CodeReuse/ActiveRecord

      def repo
        @repo ||= client.repo(project.import_source)
      end

      # import repository config items that aren't stored on the Project model
      def import_repo_config
        branch_info = repo.main_branch
        return unless branch_info['type'] == 'named_branch'
        gitlab_branch = 'branch/' + branch_info['name']
        begin
          project.change_head(gitlab_branch)
        rescue StandardError => exc
          errors << { type: :global,
                      errors: "Setting default branch to #{gitlab_branch}, got #{exc.class.name} #{exc.message}" }
        end
      end

      def import_wiki
        # gracinet: this isn't obvious, but BitbucketImport::ProjectCreator,
        # the class responsible for repo creations will creates the wiki repo
        # *only if* there's no wiki on the Bitbucket side.
        # Therefore the condition below effectively avoids a useless and
        # doomed import. This is unchanged in Heptapod at this point.
        return if project.wiki.repository_exists?

        wiki = WikiFormatter.new(project)

        project.wiki.repository.import_repository(wiki.import_url)
      rescue StandardError => e
        errors << { type: :wiki, errors: e.message }
      end

      def author_markdown(author_info)
        if author_info.nil?
          return "Anonymous"
        end

        nick = author_info['nickname']
        if nick.nil?
          return "Anonymous"
        end
        if nick == "Former user"
          # in this case author link on Bitbucket gives a useless 404
          # couldn't find better identification of former users
          return nick
        end

        display = author_info['display_name']
        label =
          if nick == display
            nick
          else
            "#{nick} (#{display})"
          end
        link = author_info.dig('links', 'html', 'href')
        if link.nil?
          label
        else
          "[#{label}](#{link})"
        end
      end

      def author_line(author_info)
        "*Created originally on Bitbucket by #{author_markdown(author_info)}*\n\n"
      end

      def logprefix
        "Bitbucket importer for #{project.namespace.path}/#{project.path} "\
        "(project id=#{project.id})"
      end

      # rubocop: disable CodeReuse/ActiveRecord
      def import_issues
        return unless repo.issues_enabled?

        Rails.logger.info("#{logprefix}: starting import of issues")
        create_labels

        client.issues(repo).each do |issue|
          gitlab_issue = project.issues.where(iid: issue.iid).first
          unless gitlab_issue.nil?
            if has_attachments(gitlab_issue)
              Rails.logger.info(
                "#{logprefix}: skipping already imported "\
                "issue #{issue.iid} having attachments on the Heptapod side")
            else
              Rails.logger.info(
                "#{logprefix}: reimport attachments if any for "\
                "issue #{issue.iid}")
              reimport_issue_attachments(issue, gitlab_issue)
            end

            next
          end

          Rails.logger.info("#{logprefix}: importing issue #{issue.iid}")
          import_issue(issue)
        end
      end

      # rubocop: disable CodeReuse/ActiveRecord
      def import_issue(issue)
        log_issue = "#{logprefix} for issue #{issue&.iid}"
        description = ''
        description += author_line(issue.author_info) unless find_user_id(issue.author)
        description += issue.description

        label_name = issue.kind
        milestone = issue.milestone ? project.milestones.find_or_create_by(title: issue.milestone) : nil

        description += import_link_issue_attachments(issue, log_issue)

        gitlab_issue = project.issues.create!(
          iid: issue.iid,
          title: issue.title,
          description: description,
          state_id: Issue.available_states[issue.state],
          author_id: gitlab_user_id(project, issue.author),
          milestone: milestone,
          created_at: issue.created_at,
          updated_at: issue.updated_at
        )

        metrics.issues_counter.increment

        gitlab_issue.labels << @labels[label_name]

        import_issue_comments(issue, gitlab_issue) if gitlab_issue.persisted?
      rescue StandardError => e
        Rails.logger.error(
          "#{log_issue}: failed with "\
          "#{e.class.name}(#{e.message})\n" + e.backtrace.join("\n"))
        errors << { type: :issue, iid: issue.iid, errors: e.message }
      end
      # rubocop: enable CodeReuse/ActiveRecord

      def import_issue_comments(issue, gitlab_issue)
        client.issue_comments(repo, issue.iid).each do |comment|
          # The note can be blank for issue service messages like "Changed title: ..."
          # We would like to import those comments as well but there is no any
          # specific parameter that would allow to process them, it's just an empty comment.
          # To prevent our importer from just crashing or from creating useless empty comments
          # we do this check.
          next unless comment.note.present?

          note = ''
          note += author_line(comment.author_info) unless find_user_id(comment.author)
          note += comment.note

          begin
            gitlab_issue.notes.create!(
              project: project,
              note: note,
              author_id: gitlab_user_id(project, comment.author),
              created_at: comment.created_at,
              updated_at: comment.updated_at
            )
          rescue StandardError => e
            errors << { type: :issue_comment, iid: issue.iid, errors: e.message }
          end
        end
      end

      # Follow redirects unless they are not on Bitbucket
      #
      # Requests to the Bitbucket API must be done with the OAuth2 token,
      # but, requests outside of Bitbucket (typically to their AWS S3 storage)
      # must not, for two reasons:
      #
      # - would leak the token
      # - AWS S3 doesn't permit coexistence of the `Authorization` header
      #   and of the auth in query string that last Bitbucket redirect provides
      #
      # Return [external?, response or URL], either
      #   - [false, response]:
      #       response would be a non streaming response from the
      #       OAuth2 library (itself relying on Faraday for HTTP)
      #   - [true, URL]
      def follow_inner_bitbucket_redirects(url)
        redir_count = 0

        while redir_count < 10
          response = client.connection.raw_get(url, false)
          case response.status
          when 301, 302, 303, 307
            redir_count += 1
            url = response.headers['location']
            Rails.logger.info("testgr: got redirection #{response.status} to #{url}")
            if url.start_with?("https://bitbucket.org")
              return [false, response]
            else
              return [true, url]
            end
          else
            return [false, response]
          end
        end
        raise AttachmentError.new("Too much redirection")
      end

      def retrieve_attachment(url, file)
        is_external, url_or_response = follow_inner_bitbucket_redirects(url)
        unless is_external
          file.write(url_or_response.body)
          return
        end

        HTTParty.get(url_or_response, stream_body: true) do |fragment|
          case fragment.code
          when 301, 302, 303, 307
            next
          when 200
            file.write(fragment)
          else
            raise AttachmentError.new("Non-success status code #{fragment.code} while streaming")
          end
        end
      end

      def import_issue_attachments(issue)
        attachments = []
        client.issue_attachments(repo, issue.iid).each do |att|
          Rails.logger.info(
            "#{logprefix}: issue #{issue.iid} "\
            "processing attachment name=#{att.name} url=#{att.retrieval_url}")
          uploader = FileUploader.new(@project, nil)
          # uploader's filename can be set, but calling `store!` on a
          # File object resets it to the basename of that file, that's
          # why we're using a tmpdir and create the file directly with
          # its target name
          Dir.mktmpdir do |dir|
            File.open(File.join(dir, att.name), 'w+b') do |file|
              retrieve_attachment(att.retrieval_url, file)
              file.seek(0)
              uploader.store!(file)
            end
          end
          attachments << uploader.to_h
        rescue StandardError => e
          Rails.logger.error("#{logprefix} failed to process attachment "\
                             "#{att.name} url=#{att.retrieval_url}"\
                             "#{e.class}: #{e}\n" + e.backtrace.join("\n"))
          errors << { type: :issue_attachment,
                      iid: issue.iid,
                      errors: "#{e.class}: #{e}",
          }
        end
        attachments
      end

      # Import attachments and return linking markdown to add to description
      def import_link_issue_attachments(issue, log_issue)
        attachments = import_issue_attachments(issue)
        return "" if attachments.empty?

        Rails.logger.info(
          "#{log_issue}: uploaded attachments #{attachments} \
                           now adding Markdown links to description")
        linking_md = "\n\n *Attachments:* "
        attachments.each do |att|
          markdown =  att[:markdown]
          if markdown.nil?
            Rails.logger.warn(
              "#{log_issue}: missing Markdown link for attachment #{att}")
            next
          end
          # Turn off automatic renderings because chances are high
          # that our rendering would not bear well with the issue
          # layout in GitLab. Users can add the exclamation mark if
          # they feel like it anyway.
          markdown = markdown[1..] if markdown[0] == '!'
          linking_md += markdown
        end
        linking_md
      end

      # Reimport attachments for an issue
      def reimport_issue_attachments(bb_issue, gitlab_issue)
        log_issue = "#{logprefix} reimport of attachments "\
                    "for issue #{bb_issue.iid}"
        linking_md = import_link_issue_attachments(bb_issue, log_issue)
        unless linking_md
          Rails.logger.info("#{log_issue} no attachments found")
          return
        end

        gitlab_issue.description += linking_md
        gitlab_issue.save!
      end

      # Rough heuristic method to tell if a GitLab issue has attachments
      #
      # Since attachments are tied only to the project, there is no
      # way to really tell, the best we can do is to look for Markdown links
      # to attachments in the description.
      #
      # If someone has removed such links manually, it can trigger reimport
      # of all attachments.
      # Conversely, if only one attachment over tens did work in a
      # previous run, nothing will be reimported.
      #
      # Only global attachments are considered.
      def has_attachments(gitlab_issue)
        match = /\[.*\]\(\/uploads/.match(gitlab_issue.description)
        !match.nil?
      end

      def create_labels
        LABELS.each do |label_params|
          label = ::Labels::FindOrCreateService.new(nil, project, label_params).execute(skip_authorization: true)
          if label.valid?
            @labels[label_params[:title]] = label
          else
            raise "Failed to create label \"#{label_params[:title]}\" for project \"#{project.full_name}\""
          end
        end
      end

      def import_pull_requests
        Rails.logger.info("#{logprefix}: starting import of Pull Requests")
        pull_requests = client.pull_requests(repo)

        pull_requests.each do |pull_request|
          import_pull_request(pull_request)
        end
      end

      def import_pull_request(pull_request)
        raw_repo = project.repository.raw_repository
        pr_iid = pull_request.iid

        log_pr = "#{logprefix} PR #{pr_iid}"

        if project.merge_requests.exists?(iid: pr_iid)
          # TODO importing new comments would make sense
          Rails.logger.info("#{log_pr} skipping already imported")
          return
        end

        pr_state = pull_request.state

        Rails.logger.info("#{log_pr}: importing (state #{pr_state})")
        description = ''
        description += author_line(pull_request.author_info) unless find_user_id(pull_request.author)

        target_branch = 'branch/' + pull_request.target_branch_name
        pr_target_sha = pull_request.target_branch_sha

        # Bitbucket gives us a sha prefix, we need to get the full one
        target_hgsha = nil
        if !pr_target_sha.nil?
          begin
            target_hgsha = raw_repo.hgsha_from_rev(pr_target_sha)
          rescue StandardError => e
            Rails.logger.error(
              "#{log_pr}: could not find target hgsha "\
              "#{pr_target_sha}, error: #{e.message}")
          end
        end

        if target_hgsha.nil?
          target_branch_sha = raw_repo.sha_from_ref(target_branch)
          target_hgsha = raw_repo.hg_sha_map[target_branch_sha]
        else
          target_branch_sha = raw_repo.git_sha_map[target_hgsha]
        end

        pr_source_sha = pull_request.source_branch_sha

        if pull_request.source_project_uuid != pull_request.target_project_uuid
          if pr_state == 'opened'
            topic = "bitbucket-pr-#{pr_iid}"
            Rails.logger.info("#{log_pr}: importing as topic #{topic}")
            source_hgsha, named_branch, topic = raw_repo.pull_force_topic(
                                          pull_request.source_project_url,
                                          pr_source_sha,
                                          topic)
            if topic.nil? || topic.empty?
              source_branch = "branch/#{named_branch}"
            else
              source_branch = "topic/#{named_branch}/#{topic}"
            end
          elsif pr_state == 'merged'
            description += "\n\n*Was already merged in Bitbucket before "
            description += "import, marked as merged "
            description += "by the import user*\n\n"
            if pr_source_sha.nil?
              description += "*Source changeset not longer available"\
                             "at the time of import. Commit list"\
                             "will look empty*\n\n"
              source_hgsha = target_hgsha
            else
              source_hgsha = raw_repo.hgsha_from_rev(pr_source_sha)
            end
            # GitLab will display the source branch as removed, and
            # is unsurprising both for declined and merged PRs.
            source_branch = "bitbucket/merged-pr-#{pr_iid}"
          else
            # Pull Request has been declined. We don't want to import the
            # commits, that's possibly harmful, some are very bogus, such
            # as PyPy's PR 64 (30000+ changesets)
            # let's make the bare minimum to have valid MR SHAs:
            # source=target
            description += "\n\n*Changesets for this Pull Request "
            description += "have not been imported, because it had been "
            description += "already declined on Bitbucket. "
            description += "Marked as closed by the import user.*\n\n"

            source_hgsha = target_hgsha

            # GitLab will display the source branch as removed, that
            # is unsurprising both for declined and merged PRs.
            source_branch = "bitbucket/declined-pr-#{pr_iid}"
          end
        else
          # Pull Request is between branches of the main Bitbucket repo
          source_hgsha = raw_repo.hgsha_from_rev(pr_source_sha)
          source_branch = 'branch/' + pull_request.source_branch_name
        end
        # it's possible that the Hg repo knows of more Git objects
        # than GitLab, because closed branch are pruned (see heptapod#168)
        # or because of changeset obsolescence.
        source_branch_sha = raw_repo.git_sha_map[source_hgsha]
        begin
          _commit = raw_repo.rev_parse_target(source_branch_sha)
        rescue StandardError => e
          source_branch_sha = target_branch_sha
          description += "*Original PR head changeset "\
                         "#{source_hgsha}) not available "\
                         "in Heptapod user interface after import. "\
                         "Displaying an "\
                         "empty list of of commits.*\n\n"
        end
        description += pull_request.description

        # TODO see how it's done more efficiently by bypassing operations
        # that are useless in import context in
        # github_importer/pull_request_importer.rb
        merge_request = project.merge_requests.create!(
          iid: pr_iid,
          title: pull_request.title,
          description: description,
          source_project: project,
          source_branch: source_branch,
          source_branch_sha: source_branch_sha,
          target_project: project,
          target_branch: target_branch,
          target_branch_sha: target_branch_sha,
          state: pull_request.state,
          author_id: gitlab_user_id(project, pull_request.author),
          assignee_id: nil,
          created_at: pull_request.created_at,
          updated_at: pull_request.updated_at
        )

        if pull_request.state == 'merged'
          # TODO it would be better to retrieve that information, but
          # we don't have it for the time being see #162
          merge_request.metrics.merged_by_id = project.creator_id
          merge_request.metrics.save
        elsif pull_request.state == 'closed'
          # TODO #162
          merge_request.metrics.latest_closed_by_id = project.creator_id
          merge_request.metrics.save
        end

        metrics.merge_requests_counter.increment

        import_pull_request_comments(pull_request, merge_request) if merge_request.persisted?
      rescue StandardError => e
        trace = e.backtrace.join("\n")
        Rails.logger.error(
          "#{log_pr} Import failed error #{e.message}, "\
          "trace: #{trace}, raw_response: #{pull_request.raw}")
        store_pull_request_error(pull_request, e)
      end

      def import_pull_request_comments(pull_request, merge_request)
        comments = client.pull_request_comments(repo, pull_request.iid)

        inline_comments, pr_comments = comments.partition(&:inline?)

        import_inline_comments(inline_comments, pull_request, merge_request)
        import_standalone_pr_comments(pr_comments, merge_request)
      end

      def import_inline_comments(inline_comments, pull_request, merge_request)
        position_map = {}
        discussion_map = {}

        children, parents = inline_comments.partition(&:has_parent?)

        # The Bitbucket API returns threaded replies as parent-child
        # relationships. We assume that the child can appear in any order in
        # the JSON.
        parents.each do |comment|
          position_map[comment.iid] = build_position(merge_request, comment)
        end

        children.each do |comment|
          position_map[comment.iid] = position_map.fetch(comment.parent_id, nil)
        end

        inline_comments.each do |comment|
          attributes = pull_request_comment_attributes(comment)
          attributes[:discussion_id] = discussion_map[comment.parent_id] if comment.has_parent?

          attributes.merge!(
            position: position_map[comment.iid],
            type: 'DiffNote')

          note = merge_request.notes.create!(attributes)

          # We can't store a discussion ID until a note is created, so if
          # replies are created before the parent the discussion ID won't be
          # linked properly.
          discussion_map[comment.iid] = note.discussion_id
        rescue StandardError => e
          errors << { type: :pull_request, iid: comment.iid, errors: e.message }
        end
      end

      def build_position(merge_request, pr_comment)
        params = {
          diff_refs: merge_request.diff_refs,
          old_path: pr_comment.file_path,
          new_path: pr_comment.file_path,
          old_line: pr_comment.old_pos,
          new_line: pr_comment.new_pos
        }

        Gitlab::Diff::Position.new(params)
      end

      def import_standalone_pr_comments(pr_comments, merge_request)
        pr_comments.each do |comment|
          merge_request.notes.create!(pull_request_comment_attributes(comment))
        rescue StandardError => e
          errors << { type: :pull_request, iid: comment.iid, errors: e.message }
        end
      end

      def pull_request_comment_attributes(comment)
        {
          project: project,
          author_id: gitlab_user_id(project, comment.author),
          note: comment_note(comment),
          created_at: comment.created_at,
          updated_at: comment.updated_at
        }
      end

      def comment_note(comment)
        author = @formatter.author_line(comment.author) unless find_user_id(comment.author)

        author.to_s + comment.note.to_s
      end

      def log_base_data
        {
          class: self.class.name,
          project_id: project.id,
          project_path: project.full_path
        }
      end

      def metrics
        @metrics ||= Gitlab::Import::Metrics.new(:bitbucket_importer, @project)
      end
    end
  end
end
