#!/usr/bin/env python3
"""Clone and prepare py_heptapod for CI build of the Rails applications.

Expected to be run from a clone of the Rails application, and assumes
that `config/gitlab.yml` has already been initialized.
"""
import os
import subprocess

INTERPRETER = 'python3.8'

CWD = os.getcwd()

PY_HEPTAPOD_CLONE_PATH = os.path.join(CWD, 'vendor', 'py-heptapod')
HGITALY_CLONE_PATH = os.path.join(CWD, 'vendor', 'hgitaly')

HG_BIN = '/usr/local/bin/hg'
PIP_BIN = 'pip3.8'

print("Upgrading pip and setuptools")
subprocess.check_call((PIP_BIN, 'install', '--upgrade',
                       'pip', 'setuptools', 'wheel'))

# yes, Mercurial is readily available in GitLab base CI image!
# But its Debian 9 version (hg 4.0)
for slug, target in (('py-heptapod', PY_HEPTAPOD_CLONE_PATH),
                     ('hgitaly', HGITALY_CLONE_PATH)):
    if os.path.exists(target):
        print("Updating %s clone at %s" % (slug, target))
        subprocess.check_call(('hg', '-R', target, 'pull', '-u'))
    else:
        print("Cloning %s into %s" % (slug, target))
        subprocess.check_call(
            ('hg', 'clone', '-q',
             'https://mirror.octobus.net/heptapod/' + slug,
             target,
             ))

    print("Current revision for %s:" % target)
    subprocess.check_call(('hg', '-R', target, 'log', '-r', '.'))

    pip_cmd = (PIP_BIN, 'install', '-e', target)
    print("Installing %s: %r" % (slug, pip_cmd))
    subprocess.check_call(pip_cmd)

pip_logging_cmd = (PIP_BIN, 'install', 'hg-loggingmod')
print("Installing hg-loggingmod: " + repr(pip_logging_cmd))
subprocess.check_call(pip_logging_cmd)

local_hgrc_path = os.path.join(CWD, 'config', 'tests.hgrc')
with open(local_hgrc_path, 'w') as local_hgrcf:
    local_hgrcf.write("\n".join((
        "[heptapod]",
        "# we won't really send the pre- and post-receive hooks, ",
        "# but `heptapod.hooks.Hook` is currently still instantiated and ",
        "# that requires the configuration.",
        "gitlab-internal-api-secret-file = " + os.path.join(CWD, '.gitlab_shell_secret'),
        "")))

req_hgrc_path = os.path.join(PY_HEPTAPOD_CLONE_PATH,
                             'heptapod', 'required.hgrc')

yml_format = 'hgrc:\n      - %s\n      - %s\n    bin_path: %s'

with open('config/gitlab.yml', 'r+') as conf:
    out_lines = [l.replace('# hgrc:',
                           yml_format % (local_hgrc_path,
                                         req_hgrc_path,
                                         HG_BIN))
                 for l in conf.readlines()]
    conf.seek(0)
    conf.writelines(out_lines)
