# coding: utf-8
# frozen_string_literal: true

require "spec_helper"

RSpec.describe Gitlab::Mercurial::HgGitRepository, :seed_helper do
  include Gitlab::EncodingHelper
  include RepoHelpers

  let(:mutable_repository) do
    repo = Gitlab::Mercurial::HgGitRepository.new('default', TEST_MUTABLE_REPO_PATH, '', 'group/project')
    repo.unit_tests_skip_hooks = true
    repo
  end
  let(:mutable_repository_path) { File.join(TestEnv.repos_path, mutable_repository.relative_path).chomp('.git') + '.hg' }
  let(:user) { build(:user) }
  let(:project) { create(:hg_git_project, :hg_repository) }
  let(:non_empty_repository) do
    repo = mutable_repository

    status = repo.create_repository(namespace_fs_path: "group",
                                    in_subgroup: false)
    expect(status).to be(true)

    repo.multi_action(user,
                      branch_name: 'branch/default',
                      message: "Commit 0",
                      author_email: "jdoe1@heptapod.test",
                      author_name: "John Doe1",
                      actions: [{ action: :create,
                                  file_path: 'foo',
                                  content: "foo0\n" }])
    repo.expire_has_local_branches_cache
    expect(repo.empty?).to be(false)
    repo
  end
  let(:non_empty_repository_path) { mutable_repository_path }

  describe "Respond to" do
    subject { mutable_repository }

    # from the base class
    it { is_expected.to respond_to(:root_ref) }
    it { is_expected.to respond_to(:tags) }

    # specific
    it { is_expected.to respond_to(:hg_sha_map) }
  end

  describe "#create_repository" do
    subject { mutable_repository }

    after do
      subject.remove
    end

    it "makes an empty repository" do
      expect(subject.create_repository(namespace_fs_path: "group",
                                       in_subgroup: false)
            ).to be(true)

      # that's from Gitaly
      expect(subject.commit_count("branch/default")).to eq(0)

      # that's from Mercurial and it implies that we have a valid repository
      expect(subject.hgsha_from_rev('tip')).to eq(Gitlab::Mercurial::NULL_NODE)
    end
  end

  describe "#rebase" do
    subject { non_empty_repository }

    after do
      subject.remove
    end

    it "is refused on named branches" do
      subject.multi_action(user,
                           branch_name: 'branch/other',
                           start_branch_name: 'branch/default',
                           message: "Commit on other branch",
                           author_email: "jdoe1@heptapod.test",
                           author_name: "John Doe1",
                           actions: [{ action: :create,
                                       file_path: 'other',
                                       content: "other foo\n" }])

      subject.multi_action(user,
                           branch_name: 'branch/default',
                           start_branch_name: 'branch/default',
                           message: "Commit on default",
                           author_email: "jdoe1@heptapod.test",
                           author_name: "John Doe1",
                           actions: [{ action: :update,
                                       file_path: 'foo',
                                       content: "foo1\n" }])

      expect do
        subject.rebase(nil, 1,
                       branch: 'branch/other',
                       branch_sha: subject.hgsha_from_rev('other'),
                       remote_repository: subject,
                       remote_branch: 'branch/default')
      end.to raise_error(Gitlab::Mercurial::HgGitRepository::HgError)
    end
  end

  describe "publish permission" do
    # we don't need to add `user` to the project, because write
    # authorization is entirely managed at higher level (Models).
    # Should that change, then the test with topic creation would fail
    subject { project.repository.raw_repository }

    context "when hg_project_for_perms is provided" do

      it "is enforced automatically" do
        # we're using #multi_action for an easy check, but the point
        # is that it happens without direct involvement of the method
        expect do
          subject.multi_action(user,
                               branch_name: 'branch/default',
                               start_branch_name: 'branch/default',
                               message: "Public commit on default",
                               author_email: "jdoe2@heptapod.test",
                               author_name: "John Doe2",
                               actions: [{ action: :create,
                                           file_path: 'foo',
                                           content: "Cannot publish that\n" }])
        end.to raise_error(Gitlab::Mercurial::HgGitRepository::HgError)
      end

      it "allows creating draft changesets" do
        expect do
          subject.multi_action(user,
                               branch_name: 'topic/default/user',
                               start_branch_name: 'branch/default',
                               message: "Topical changeset",
                               author_email: "jdoe2@heptapod.test",
                               author_name: "John Doe2",
                               actions: [{ action: :create,
                                           file_path: 'foo',
                                           content: "Can commit that\n" }])
        end.not_to raise_error
      end

      it "lets owner publish changesets" do
        expect do
          subject.multi_action(project.creator,
                               branch_name: 'branch/default',
                               start_branch_name: 'branch/default',
                               message: "New changeset",
                               author_email: "jdoe1@heptapod.test",
                               author_name: "John Doe1",
                               actions: [{ action: :create,
                                           file_path: 'bar',
                                           content: "Can publish\n" }])
        end.not_to raise_error
      end
    end
  end

end
